<select name="{{ name }}" id="{{ id }}">
  <option value="">Select Columns</option>
  {% for item in items %}
    <option value="{{ item }}">{{ item }}</option>
  {% endfor %}
</select>

{{ item.render() }}