<a class="anchor-pos" id="top"></a>
<nav class="navbar navbar-default navbar-fixed-top">
    <div class="container-fluid">
        <div class="navbar-header">
            <button type="button"
                    class="navbar-toggle collapsed"
                    data-toggle="collapse"
                    data-target="#navbar"
                    aria-expanded="false"
                    aria-controls="navbar">
                <span class="sr-only">Toggle navigation</span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
            </button>
            {% if logo | length > 0 %}
                <a class="navbar-brand" href="#top">
                    <img alt="Logo" src="{{ logo }}" style="max-height: 20px;">
                </a>
            {% endif %}
            <a class="navbar-brand anchor" href="#top">{{ title }}</a>
        </div>
        <div id="navbar" class="navbar-collapse collapse">
            <ul class="nav navbar-nav navbar-right">
                {% for (name, anchor_id) in nav_items %}
                    <li>
                        <a href="#{{ anchor_id }}">{{ name }}</a>
                    </li>
                {% endfor %}
            </ul>
        </div>
    </div>
</nav>
